@@ -4,7 +4,6 @@
 #' 
 #' @name BreedingSchemeLanguage
 #' @docType package
-#' @import ggplot2 lme4 snowfall Rcpp 
 #' @useDynLib BreedingSchemeLanguage
 #' @importFrom Rcpp sourceCpp
-library(Rcpp)
+#' @importFrom snowfall sfLapply sfInit sfStop
