@@ -1,26 +1,21 @@
 #'Select individuals
 #'
-#'@param sEnv the environment that BSL functions operate in. Default is "simEnv" so use that to avoid specifying when calling functions
-#'@param nSelect the number of selected individuals
+#'@param sEnv the environment that BSL functions operate in. If NULL, the default \code{simEnv} is attempted
+#'@param nSelect the number of selected individuals (default: 40)
 #'@param popID population ID to be selected (default: When random=T, the last population. When random=F, it is the last evaluated population)
 #'@param random assuming random selection or selection according to estimated value (T: random selection, F: selection for high value)
-#'@param type "WithinFamily" or "Mass" (default: Mass). If Mass, all individuals are ranked against each other and the highest nSelect are taken.  If WithinFamily, individuals are ranked within paternal half-sib (if population was randomly mated) or full-sib (if population from selfFertilize or doubledHaploid) families. If random=T, then selection within families is random.
-#'@param parms an optional named list or vector. Objects with those names will be created with the corresponding values. A way to pass values that are not predetermined by the script.
+#'@param type "WithinFamily" or "Mass" (default: Mass). If Mass, all individuals are ranked against each other and the highest nSelect are taken.  If WithinFamily, individuals are ranked within paternal half-sib (if population was randomly mated) or full-sib (if population from selfFertilize or doubledHaploid) families. If random=T, then mass or within-family selection is random.  NOTE: breeding scheme budget may not be correct for within-family selection because then nSelect is per-family but the number of families varies across simulations.
 #'
 #'@seealso \code{\link{defineSpecies}} for an example
 #'
 #'@return modifies the list sims in environment sEnv by selecting individuals of the specified popID with the default selection criterion and giving those individuals a new popID
 #'
 #'@export
-select <- function(sEnv=NULL, nSelect=40, popID=NULL, random=F, type="Mass", parms=NULL){
-  if(!is.null(parms)){
-    for (n in 1:length(parms)){
-      assign(names(parms)[n], parms[[n]])
-    }
-  }
+select <- function(sEnv=NULL, nSelect=40, popID=NULL, random=F, type="Mass"){
   select.func <- function(bsl, nSelect, popID, random, type){
     criterion <- bsl$selCriterion$criterion
-    if(is.null(popID)){
+    if (is.null(criterion)) random <- TRUE
+    if (is.null(popID)){
       popID <- bsl$selCriterion$popID
       if(is.null(popID)) popID <- 0
     }
@@ -46,8 +41,12 @@ select <- function(sEnv=NULL, nSelect=40, popID=NULL, random=F, type="Mass", par
       if(substr(criterion, 1, 4) == "pred"){
         GIDcan <- intersect(GIDcan, bsl$predRec$predGID)
         if (length(GIDcan) == 0) stop(paste("There are no selection candidates in the population", popID, "with predictions"))
-        usePred <- bsl$predRec[bsl$predRec$predGID %in% GIDcan & bsl$predRec$predNo == max(bsl$predRec$predNo),]
-        candValue <- usePred$predict[order(usePred$predGID)]
+        # Return the last prediction for a candidate
+        lastPred <- function(cand){
+          usePred <- bsl$predRec[bsl$predRec$predGID == cand,,drop=F]
+          return(usePred[which.max(usePred$predNo), "predict"])
+        }
+        candValue <- sapply(GIDcan, lastPred)
       }
       # Done computing the candidate values
       if (type == "WithinFamily"){
@@ -61,9 +60,8 @@ select <- function(sEnv=NULL, nSelect=40, popID=NULL, random=F, type="Mass", par
         selectedGID <- GIDcan[order(candValue, decreasing=T)[1:nSelect]]
       }
     }#END not random selection
-    popID.new <- max(bsl$genoRec$popID) + 1
-    bsl$genoRec$popID[bsl$genoRec$GID %in% selectedGID] <- popID.new
-    if (exists("totalCost", bsl)) bsl$totalCost <- bsl$totalCost + bsl$costs$selectCost
+    popID_new <- max(bsl$genoRec$popID) + 1
+    bsl$genoRec$popID[bsl$genoRec$GID %in% selectedGID] <- popID_new
     return(bsl)
   } #END select.func
   
@@ -76,12 +74,23 @@ select <- function(sEnv=NULL, nSelect=40, popID=NULL, random=F, type="Mass", par
   } 
   parent.env(sEnv) <- environment()
   with(sEnv, {
-    if(nCore > 1){
-      snowfall::sfInit(parallel=T, cpus=nCore)
-      sims <- snowfall::sfLapply(sims, select.func, nSelect=nSelect, popID=popID, random=random, type=type)
-      snowfall::sfStop()
-    }else{
-      sims <- lapply(sims, select.func, nSelect=nSelect, popID=popID, random=random, type=type)
+    if (exists("totalCost")){
+      costsPopID <- ifelse(is.null(popID), ifelse(is.null(costs$popID), 0, costs$popID), popID)
+      toSample <- budgetRec$GID[budgetRec$popID %in% costsPopID]
+      GIDsel <- sample(toSample, min(nSelect, length(toSample)))
+      budgetRec$popID[budgetRec$GID %in% GIDsel] <- max(budgetRec$popID) + 1
+      totalCost <- totalCost + costs$selectCost
+      rm(costsPopID, GIDsel)
+    }
+    
+    if (!onlyCost){
+      if(nCore > 1){
+        snowfall::sfInit(parallel=T, cpus=nCore)
+        sims <- snowfall::sfLapply(sims, select.func, nSelect=nSelect, popID=popID, random=random, type=type)
+        snowfall::sfStop()
+      }else{
+        sims <- lapply(sims, select.func, nSelect=nSelect, popID=popID, random=random, type=type)
+      }
     }
   })
 }
